@@ -48,7 +48,10 @@ def repository_handler(repoent):
     except KeyError:
         pass
     try:
-        repohdlr = _VCSTYPES[repoent.type](repoent.eid, repoent.path, repoent.encoding)
+        path = repoent.path
+        if not path:
+            path = repoent.local_cache
+        repohdlr = _VCSTYPES[repoent.type](repoent.eid, path, repoent.encoding)
     except KeyError:
         msg = repoent._cw._('%s is not a known repository type')
         raise ValidationError(repoent.eid, {'type': msg % repoent.type})
@@ -59,36 +62,32 @@ def repository_handler(repoent):
     return repohdlr
 
 
-def import_content(repo, commitevery=10):
+def import_content(session, repo, commitevery=10):
     """synchronize content of known vcs repositories
 
     `repo` is the cubicweb repository (not a vcs repository)
     """
-    session = repo.internal_session()
-    try:
-        for repoentity in session.execute(
-            'Any X,T,P,SP,E WHERE X is Repository, '
-            'X type T, X path P, X subpath SP, X encoding E').entities():
-            # give a change to threads waiting for a pool
-            session.set_pool()
+    for repoentity in session.execute(
+        'Any X,T,P,SP,E WHERE X is Repository, '
+        'X type T, X path P, X subpath SP, X encoding E').entities():
+        # give a change to threads waiting for a pool
+        session.set_pool()
+        try:
+            repohdlr = repository_handler(repoentity)
+        except VCSException, ex:
+            repo.error(str(ex))
+            continue
+        try:
+            repohdlr.import_content(repoentity, commitevery)
+            session.commit()
+        except:
             try:
-                repohdlr = repository_handler(repoentity)
-            except VCSException, ex:
-                repo.error(str(ex))
-                continue
-            try:
-                repohdlr.import_content(repoentity, commitevery)
-                session.commit()
+                title = repoentity.dc_title()
             except:
-                try:
-                    title = repoentity.dc_title()
-                except:
-                    title = str(repoentity)
-                repoentity.exception(
-                    'error while importing content for vcs repo %s', title)
-                session.rollback()
-    finally:
-        session.close()
+                title = str(repoentity)
+            repoentity.exception(
+                'error while importing content for vcs repo %s', title)
+            session.rollback()
 
 
 def set_at_revision(session, reid, safetybelt=False):
