@@ -5,12 +5,14 @@
 :contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
 """
 __docformat__ = "restructuredtext en"
+import os.path as osp
 
 from logilab.common.decorators import cached, clear_cache
 from logilab.common.compat import any
 
 from cubicweb import QueryError, Binary, Unauthorized
-from cubicweb.entities import AnyEntity, fetch_config
+from cubicweb.selectors import is_instance
+from cubicweb.entities import AnyEntity, fetch_config, adapters
 
 from cubes.vcsfile import queries
 
@@ -44,6 +46,8 @@ class Repository(AnyEntity):
     def dc_title(self):
         if self.title:
             return self.title
+        if self.source_url:
+            return self.source_url
         if self.path: # None if user isn't authorized to see path attribute
             title = '%s:%s' % (self.type, self.path)
             if self.subpath:
@@ -68,6 +72,14 @@ class Repository(AnyEntity):
             return -1
         return latestrev
 
+    def heads_rset(self):
+        """return a result set containing head revision (eg revisions without
+        children)
+        """
+        return self._cw.execute(
+            'Any REV WHERE R eid %(r)s, REV from_repository R, '
+            'NOT CREV parent_revision REV', {'r' : self.eid})
+
     def branches(self):
         """return existing branches"""
         return [b for b, in self._cw.execute(
@@ -408,7 +420,6 @@ class DeletedVersionContent(AnyEntity):
         return bridge.repository_handler(vcsrepo), transaction
 
 
-from cubicweb import QueryError
 from cubes.vcsfile import bridge
 try:
     from cubes.vcsfile.hooks import missing_relation_error, CheckRevisionOp
@@ -437,3 +448,18 @@ class VersionContent(DeletedVersionContent):
     def size(self):
         return self._cw.execute("Any LENGTH(D) WHERE X eid %(eid)s, X data D",
                                 {'eid': self.eid})[0][0]
+
+
+class VersionContentIDownloadableAdapter(adapters.IDownloadableAdapter):
+    __select__ = is_instance('VersionContent')
+
+    def download_url(self, **kwargs):
+        return self.entity.absolute_url(vid='download', **kwargs)
+    def download_content_type(self):
+        return self.entity.data_format
+    def download_encoding(self):
+        return self.entity.data_encoding
+    def download_file_name(self):
+        return self.entity.file.name
+    def download_data(self):
+        return self.entity.data.getvalue()
