@@ -8,16 +8,39 @@
 __docformat__ = "restructuredtext en"
 
 import os
+import os.path as osp
+import shutil
 
 from logilab.mtconverter import need_guess, guess_mimetype_and_encoding
 
 from cubicweb import QueryError, ValidationError, Binary
 from cubicweb.server import hook
 from cubicweb.server.hook import Hook, Operation, set_operation
-from cubicweb.selectors import is_instance
+from cubicweb.selectors import is_instance, score_entity
 
 from cubes.vcsfile import IMMUTABLE_ATTRIBUTES, bridge
 
+def repo_cache_dir(config):
+    directory = config['local-repo-cache-root']
+    if not osp.isabs(directory):
+        directory = osp.join(config.appdatahome, directory)
+    if not osp.exists(directory):
+        try:
+            os.makedirs(directory)
+        except:
+            config.critical('could not find local repo cache directory; check '
+                            'local-repo-cache-root option whose present value is %r)',
+                            config['local-repo-cache-root'])
+            raise
+    return directory
+
+def url_to_relative_local_path(url):
+    scheme, url = url.split('://', 1)
+    if scheme == 'file':
+        return url.rsplit('/', 1)[-1]
+    else:
+        return url.rsplit('/', 1)[1]
+
 def missing_relation_error(entity, rtype):
     # use __ since msgid recorded in cw, we don't want to translate it in
     # this cube
@@ -28,22 +51,70 @@ def missing_relation_error(entity, rtype):
                                         'eid': entity.eid}}
     return ValidationError(entity.eid, errors)
 
+def missing_attribute_error(entity, attrs):
+    # use __ since msgid recorded in cw, we don't want to translate it in
+    # this cubex
+    __ = entity._cw._
+    msg = _('at least one attribute of %s must be set on a Repository')
+    errors = {}
+    for attr in attrs:
+        errors[attr] = msg % ', '.join(attrs)
+    return ValidationError(entity.eid, errors)
+
 # initialization hooks #########################################################
 
+class ServerMaintenanceHook(Hook):
+    """install attribute map on the system source sql generator
+    """
+    __regid__ = 'vcsfile.servermaintenancehook'
+    events = ('server_maintenance',)
+    def __call__(self):
+        repo = self.repo
+        repo.system_source.set_storage('VersionContent', 'data',
+                                       bridge.VCSStorage())
+
+
 class ServerStartupHook(Hook):
     """synchronize repository on server startup, and install attribute map
     on the system source sql generator
     """
     __regid__ = 'vcsfile.serverstartuphook'
-    events = ('server_startup', 'server_maintenance')
+    events = ('server_startup',)
     def __call__(self):
         repo = self.repo
         repo.system_source.set_storage('VersionContent', 'data',
                                        bridge.VCSStorage())
-        if self.event == 'server_startup' and repo.config['repository-import']:
-            repo.looping_task(repo.config.get('check-revision-interval', 5*60),
-                              bridge.import_content, repo,
-                              repo.config.get('check-revision-commit-every', 1))
+        cacheroot = repo_cache_dir(repo.config)
+        if not os.access(cacheroot, os.W_OK):
+            raise ValueError, ('directory %r is not writable '
+                               '(check  option)' % (cacheroot, CACHEROOT))
+
+        def refresh_local_hgrepo_caches(repo, osp=osp, bridge=bridge, cacheroot=cacheroot):
+            session = repo.internal_session()
+            try:
+                reposrset = session.execute('Any R, RT, RLC, RSU, RP WHERE R is Repository, '
+                                            'R type "mercurial", NOT R local_cache NULL, '
+                                            'R type RT, R local_cache RLC, '
+                                            'R source_url RSU, R path RP')
+                for repository in reposrset.entities():
+                    repo.info('refreshing local repository caches for %s', repository.eid)
+                    # do stuff with repositories in the cacheroot
+                    repohandler = bridge.repository_handler(repository)
+                    if not osp.exists(repository.local_cache):
+                        repo.info('cloning into local cache %s', repository.source_url)
+                        repohandler.clone_to_cache(repository)
+                    else:
+                        repohandler.pull_on_cache(repository)
+                    # give other threads a chance to run
+                    session.set_pool()
+                if repo.config['repository-import']:
+                    bridge.import_content(session, repo, repo.config.get('check-revision-commit-every', 1))
+            finally:
+                session.close()
+
+        repo.looping_task(repo.config.get('check-revision-interval'),
+                          refresh_local_hgrepo_caches, repo)
+
 
 # internals to be able to create new vcs repo revision using rql queries #######
 
@@ -51,9 +122,12 @@ class VCTransactionOp(Operation):
 
     def precommit_event(self):
         bridge.set_at_revision(self.session, self.revision.eid)
+        transactions = self.session.transaction_data.setdefault('vctransactions', {})
+        for transaction in transactions.itervalues():
+            transaction.precommit()
 
     def revertprecommit_event(self):
-        transactions = self.session.transaction_data.setdefault('vctransactions', {})
+        transactions = self.session.transaction_data.pop('vctransactions', {})
         for transaction in transactions.itervalues():
             transaction.rollback()
 
@@ -62,21 +136,18 @@ class VCTransactionOp(Operation):
         for transaction in transactions.itervalues():
             transaction.commit()
 
-    revertcommit_event = revertprecommit_event
-
     def rollback_event(self):
         transactions = self.session.transaction_data.setdefault('vctransactions', {})
         for transaction in transactions.itervalues():
             transaction.rollback()
 
 
-
-
 class VFModificationDateOp(Operation):
     def precommit_event(self):
         for entity in self.session.transaction_data.pop('vcsfile.modifiedvfs'):
             entity.file.set_attributes(modification_date=entity.rev.creation_date)
 
+
 class UpdateVFModificationDateHook(Hook):
     __regid__ = 'vcsfile.update_vf_mdate'
     events = ('after_add_entity',)
@@ -172,6 +243,80 @@ class AddVersionedFileHook(Hook):
             return
         CheckVersionedFileOp(self._cw, entity=self.entity)
 
+
+class SetRepoCacheBeforeAdd(Hook):
+    """clone just after repo creation
+    mostly helps the tests
+    """
+    __regid__ = 'clone_hgrepo_after_add'
+    __select__ = (Hook.__select__ & is_instance('Repository'))
+    events = ('before_add_entity',)
+    order = 2
+    def __call__(self):
+        repo = self.entity
+        if repo.type == 'mercurial' and repo.source_url and not repo.path:
+            cachedir = osp.join(repo_cache_dir(self._cw.vreg.config),
+                                unicode(repo.eid))
+            if not osp.exists(cachedir):
+                try:
+                    os.makedirs(cachedir)
+                except OSError:
+                    self.exception('cant create repo cache directory %s', cachedir)
+                    return
+            try:
+                repo['local_cache'] = osp.join(
+                    cachedir, url_to_relative_local_path(repo.source_url))
+            except (IndexError, ValueError):
+                # do nothing, validation error should be raised by another hook
+                pass
+
+class CloneRepoAfterAdd(Hook):
+    """clone just after repo creation, to allow operation which needs local
+    access and get quicker checkout if test are running on the same host
+    """
+    __regid__ = 'clone_hgrepo_after_add'
+    __select__ = (Hook.__select__ & is_instance('Repository'))
+    events = ('after_add_entity',)
+    order = 2
+
+    def __call__(self):
+        repository = self.entity
+        if not repository.local_cache:
+            return
+        self.info('cloning into local cache %s', repository.source_url)
+        repohandler = bridge.repository_handler(repository)
+        try:
+            # may already exist during test
+            if not osp.exists(repository.local_cache):
+                repohandler.clone_to_cache(repository)
+            else:
+                repohandler.pull_on_cache(repository)
+        except:
+            self.exception('while trying to clone repo %s',
+                           repository.source_url)
+            msg = self._cw._('cannot clone the repo from %s, please check the source url')
+            msg %= repository.source_url
+            raise ValidationError(repository.eid, {'source_url': msg})
+
+
+class DeleteRepositoryHook(Hook):
+    __regid__ = 'vcsfile.add_update_repository_hook'
+    __select__ = (Hook.__select__ &
+                  is_instance('Repository') &
+                  score_entity(lambda x:x.local_cache))
+    events = ('before_delete_entity',)
+
+    def __call__(self):
+        set_operation(self._cw, 'delete_repo_cleanup', self.entity.local_cache, DeleteRepositoryOp)
+
+class DeleteRepositoryOp(Operation):
+
+    def postcommit_event(self):
+        session = self.session
+        for cache in session.transaction_data.pop('delete_repo_cleanup'):
+            self.info('deleting repository cache at %s', cache)
+            shutil.rmtree(cache.encode('ascii'))
+
 # safety belts #################################################################
 
 def _check_in_transaction(vf_or_rev):
@@ -211,7 +356,7 @@ class CheckRevisionOp(Operation):
             raise QueryError('entity linked to a bad revision')
 
 
-class CheckImmutalbeAttributeHook(Hook):
+class CheckImmutableAttributeHook(Hook):
     __regid__ = 'vcsfile.check_immutable_attribute_hook'
     events = ('before_update_entity',)
     __select__ = Hook.__select__ & is_instance('Revision', 'DeletedVersionContent',
@@ -230,22 +375,48 @@ class UpdateRepositoryHook(Hook):
     __regid__ = 'vcsfile.update_repository_hook'
     events = ('before_update_entity', )
     __select__ = Hook.__select__ & is_instance('Repository')
+    category = 'vcsfile.repo_immutable_attrs'
+
     def __call__(self):
         # XXX check value actually changed
         try:
             edited = self.entity.edited_attributes
         except AttributeError:
             edited = self.entity
-        if 'path' in edited:
-            msg = self._cw._('updating path attribute of a repository isn\'t '
-                            'supported. Delete it and add a new one.')
-            raise QueryError(msg)
-        if 'type' in edited:
-            msg = self._cw._('updating type attribute of a repository isn\'t '
-                            'supported. Delete it and add a new one.')
-            raise QueryError(msg)
+        errors = []
+        for attr in ('path', 'type', 'source_url'):
+            if attr in edited:
+                msg = self._cw._('updating %s attribute of a repository isn\'t '
+                                 'supported. Delete it and add a new one.') %  attr
+                errors.append((attr, msg))
+        if errors:
+            raise ValidationError(self.entity.eid, dict(errors))
 
 
+class AddUpdateRepositoryHook(Hook):
+    __regid__ = 'vcsfile.add_update_repository_hook'
+    __select__ = Hook.__select__ & is_instance('Repository')
+    events = ('after_add_entity', 'after_update_entity')
+    required_attrs = ('path', 'source_url')
+    order = 1
+
+    def __call__(self):
+        entity = self.entity
+        if entity.path and not osp.isabs(self.entity.path):
+            msg = self._cw._('path must be absolute')
+            raise ValidationError(self.entity.eid, {'path': msg})
+        if entity.type == 'subversion' and not entity.path:
+            msg = self._cw._('path is mandatory for subversion repositories')
+            raise ValidationError(entity.eid, {'path': msg})
+        if not entity.source_url and not entity.path:
+            raise missing_attribute_error(entity, self.required_attrs)
+        if entity.source_url and not entity.path and not entity.local_cache:
+            try:
+                url_to_relative_local_path(entity.source_url)
+            except:
+                msg = self._cw._('url must be of the form protocol://path/to/stuff')
+                raise ValidationError(entity.eid, {'source_url': msg})
+
 # folder/tag extensions ########################################################
 
 class ClassifyVersionedFileHook(Hook):
