@@ -7,10 +7,10 @@
 __docformat__ = "restructuredtext en"
 
 from os import umask
-from os.path import split
+from os.path import split, exists
 from datetime import datetime
 
-from mercurial import hg, ui, revlog, context, util
+from mercurial import hg, ui, revlog, context, util, commands as hgcmd
 from mercurial.node import nullid, short as short_hex
 
 from cubicweb import Binary, QueryError
@@ -19,6 +19,27 @@ from cubes.vcsfile import repo, bridge
 
 _DELETED = object()
 
+class vcsfile_ui(ui.ui):
+    """ redirect output to cubicweb log """
+
+    def copy(self):
+        return self
+
+    def interactive(self):
+        return False
+
+    # XXX beware encoding issues wrt localized outputs ...
+    def write(self, *args, **kwargs):
+        if self._buffers:
+            self._buffers[-1].extend([str(a) for a in args])
+        else:
+            for a in args:
+                self.cw_loggable.debug(str(a).strip())
+
+    def write_err(self, *args, **kwargs):
+        for a in args:
+            self.cw_loggable.debug(str(a).strip())
+
 
 def bw_nb_revisions(changelog):
     """return the number of revisions in a repository's changelog,
@@ -33,9 +54,27 @@ def bw_nb_revisions(changelog):
 class HGRepository(repo.VCSRepository):
     type = 'hg'
 
+    def clone_to_cache(self, repoent):
+        vui = vcsfile_ui()
+        vui.cw_loggable = repoent
+        hgcmd.clone(vui,
+                    repoent.source_url.encode('ascii'),
+                    dest=repoent.local_cache.encode('ascii'),
+                    stream='compressed',
+                    noupdate=True)
+
+    def pull_on_cache(self, repoent):
+        lui = vcsfile_ui()
+        lui.cw_loggable = repoent
+        hgrepo = hg.repository(lui, repoent.local_cache)
+        hgcmd.pull(lui, hgrepo, source=repoent.source_url.encode('ascii'))
+
     def import_content(self, repoentity, commitevery=10):
         """import content from the mercurial repository"""
         assert self.eid == repoentity.eid
+        if not exists(repoentity.path or repoentity.local_cache):
+            self.info('repository %s not accessible', repoentity.dc_title())
+            return
         try:
             repo = self.hgrepo()
         except Exception, ex:
@@ -46,24 +85,25 @@ class HGRepository(repo.VCSRepository):
         actualrev = bw_nb_revisions(repo.changelog) - 1
         session = repoentity._cw
         latestrev = repoentity.latest_known_revision()
+        repopath = repoentity.path or repoentity.local_cache
         if actualrev < latestrev:
             # striped repo
             # very minimal support, this won't work if local number of
             # revisions has changed due to the strip
             self.warning('%s: strip detected ? latest known revision is %s, latest '
-                         'repository revision is %s', repoentity.path, latestrev, actualrev)
+                         'repository revision is %s', repopath, latestrev, actualrev)
             session.execute('DELETE Revision X WHERE X revision > %(rev)s',
                             {'rev': actualrev})
             return
         self.info('hg repo %s: known revisions up to %s, repo revision %s',
-                  repoentity.path, latestrev, actualrev)
+                  repopath, latestrev, actualrev)
         for i in xrange(latestrev + 1, actualrev + 1):
             self.info('importing revision %s', i)
             try:
                 self.import_revision(repoentity, repo, i)
             except:
                 self.critical('error while importing revision %s of %s',
-                              i, repoentity.path, exc_info=True)
+                              i, repopath, exc_info=True)
                 raise
             if not i % commitevery:
                 # give a change to threads waiting for a pool by freeing it then
@@ -211,9 +251,11 @@ class HGTransaction(object):
             raise IOError()
         return self.changes[path]
 
-    def commit(self):
+    def precommit(self):
         if not self.changes:
             raise QueryError('nothing changed')
+
+    def commit(self):
         # XXX: make umask configurable
         oldumask = umask(022)
         try:
@@ -271,7 +313,7 @@ class HGTransaction(object):
             # mercurial >= 1.3.1
             self._wlock.release()
             self._lock.release()
-        except:
+        except AttributeError:
             del self._wlock
             del self._lock
 
