@@ -32,13 +32,19 @@ class Repository(EntityType):
         'update': ('managers',),
         'delete': ('managers',),
         }
+
     title = String(description=_('optional title. If the repository has a public url, '
                                  'it may be a good idea to put it as title.'))
     type = String(required=True, vocabulary=('subversion', 'mercurial'),
                   description=_('repository\'s type'))
-    path = String(required=True, unique=True,
-                  description=_('path to the repository'),
+    source_url = String(description=_('source url (must be of the form protocol://path/to/stuff)'),
+                        unique=True)
+    path = String(unique=True,
+                  description=_('absolute path to the local repository'),
                   __permissions__=PRIVATE_ATTRPERMS)
+    local_cache = String(unique=True,
+                         description=_('absolute path to the local cache of the repository'),
+                         __permissions__=PRIVATE_ATTRPERMS)
     subpath = String(description=_('if specified, only import the given subpart'
                                    ' of the repository (path relative to the '
                                    'repository root)'))
