@@ -41,5 +41,13 @@ cause performance issue when scaling.',
           'level': 2,
           'group': 'vcsfile',
           }),
+        ('local-repo-cache-root',
+         {'type':'string',
+          'default': 'repo_cache', # XXX /var/lib/cubicweb/<instance>
+          'help':'Local repository cache location (if not absolute, will be '
+          'relative to instance data home directory).',
+          'level': 2,
+          'group': 'vcsfile'
+          }
+         ),
         )
-
