@@ -2,6 +2,7 @@
 import os
 from logilab.common.testlib import unittest_main
 from cubicweb.devtools import BaseApptestConfiguration, testlib
+from utils import HERE, osp
 
 class MyConfig(BaseApptestConfiguration):
     sourcefile = 'sources'
@@ -15,7 +16,8 @@ class AutomaticWebTest(testlib.AutomaticWebTest):
 
     def custom_populate(self, how_many, cursor):
         self.request().create_entity('Repository', type=u'mercurial',
-                                     path=u'testrepohg', encoding=u'latin1')
+                                     path=unicode(osp.abspath(osp.join(HERE, 'testrepohg'))),
+                                     encoding=u'latin1')
 
 if __name__ == '__main__':
     unittest_main()
