@@ -99,14 +99,14 @@ class VFViewsTC(VCSFileTC):
         self.assertEquals(toto.directory, "")
         self.assertEquals(toto.path, "toto.txt")
         self.assertEquals(toto.head.revision, 2)
-        self.assertEquals(toto.repository.path, 'testrepo')
+        self.assertTrue(toto.repository.path.endswith('testrepo'))
         tutu = self.execute('VersionedFile X WHERE X name %(name)s', {'name': 'tutu.txt'}).get_entity(0, 0)
         self.assertIsInstance(tutu, entities.VersionedFile)
         self.assertEquals(tutu.name, "tutu.txt")
         self.assertEquals(tutu.directory, "dir1")
         self.assertEquals(tutu.path, "dir1/tutu.txt")
         self.assertEquals(tutu.head.revision, 4)
-        self.assertEquals(tutu.repository.path, 'testrepo')
+        self.assertTrue(tutu.repository.path.endswith('testrepo'))
         self.assertEquals(tutu.deleted_in_branch(), False)
         data = self.execute('VersionedFile X WHERE X name %(name)s', {'name': 'data.bin.gz'}).get_entity(0, 0)
         self.assertEquals(data.head.revision, 3)
