@@ -26,9 +26,15 @@ from cubes.vcsfile import IMMUTABLE_ATTRIBUTES
 _afs = uicfg.autoform_section
 _aff = uicfg.autoform_field
 _affk = uicfg.autoform_field_kwargs
+_rctl = uicfg.reledit_ctrl
 
+for attr in ('path', 'subpath', 'type', 'source_url', 'encoding'):
+    _rctl.tag_attribute(('Repository', attr), {'noedit': True})
+_rctl.tag_attribute(('Repository', 'title'), {'reload': True})
+_affk.tag_attribute(('Repository', 'source_url'), {'widget': fw.TextInput})
 _affk.tag_attribute(('Repository', 'path'), {'widget': fw.TextInput})
 _affk.tag_attribute(('Repository', 'subpath'), {'widget': fw.TextInput})
+_afs.tag_attribute(('Repository', 'local_cache'), 'main', 'hidden')
 
 _afs.tag_attribute(('Revision', 'revision'), 'main', 'hidden')
 _afs.tag_attribute(('Revision', 'changeset'), 'main', 'hidden')
